@@ -38,7 +38,10 @@ exp:
         { $_[0]->{out}= { op1 => 'call', sub => $_[1], } }
     | BAREWORD exp   %prec P001
         { $_[0]->{out}= { op1 => 'call', sub => $_[1], param => $_[2], } }
-        
+ 
+    | REDUCE exp   %prec P001
+        { $_[0]->{out}= { %{$_[1]}, param => $_[2], } }
+       
     | DOT_BAREWORD exp   %prec P001
         { $_[0]->{out}= { op1 => 'method_call', self => { 'scalar' => '$_' }, method => $_[1], param => $_[2], } }
     | DOT_BAREWORD   
