@@ -8,16 +8,16 @@ sub true is export {
     $_[0] ?? bool::True !! bool::False
 }
 
-sub Inf { Pugs::Runtime::Perl6::Inf }
-sub NaN { Pugs::Runtime::Perl6::NaN }
+sub Inf is export { Pugs::Runtime::Perl6::Inf }
+sub NaN is export { Pugs::Runtime::Perl6::NaN }
 
 sub slurp { ... } # Slurps in an entire file
 
 module Perl6::Internals;
 
 sub compile_pir($code) { ... }
-sub exit($status)   { use v5; exit($status); use v6; }
-sub sleep($seconds) { use v5; sleep($seconds); use v6; }
+sub exit($status)   is export { use v5; CORE::exit($status); use v6; }
+sub sleep($seconds) is export { use v5; CORE::sleep($seconds); use v6; }
 
 
 module Pugs::Internals;
